FUNCTION abund,solar=solar,angr=angr,feld=feld,aneb=aneb,mmc=mmc
;+
; NAME: abund
;
;
;
; PURPOSE: return elemental abundances for the important elements
;
;
;
; CATEGORY: astronomy
;
;
;
; CALLING SEQUENCE: ab=abund(solar=solar,angr=angr,feld=feld,aneb=aneb,mmc=mmc)
;
;
;
; INPUTS: none
;
;
;
; OPTIONAL INPUTS: none
;
;
;
; KEYWORD PARAMETERS:
;   solar     : Standard solar values after Wilms, Allen, and McCray
;   angr      : Anders & Grevesse, 1989, Geochim. Cosmochim. Acta 53, 197
;   aneb      : Anders & Ebihara 1982, Geochim. Cosmochim. Acta 46, 2363
;   feld      : Feldman, 1992, Physica Scripta 46, 202
;      (last three copied from XSPEC code, no grain depletions)
;   mmc       : Morrison & McCammon, 1983, ApJ 270, 119 (including grains)
;             (essentially Anders and Ebihara, but H:He = 10:1)
;
;
;
; OUTPUTS:
;
;
; The function returns the abundances and depletion factors for 
; important elements in the ISM
;
; no keyword: Standard ISM values after Wilms, Allen, and McCray
;
; In each row the array returned contains:
;    1. The nuclear charge of the element,
;    2. The log of the abundance in number wrt H
;    3. The depletion factor 1-beta (i.e., the ratio of the GAS
;       to the total ISM abundance
;   4. The atomic weight of the element
; 
; MODIFICATION HISTORY:
;
; $Log: abund.pro,v $
; Revision 1.2  2003/07/10 15:34:19  wilms
; repaired stupid mistake of mine...
;
; Revision 1.1  2003/07/10 13:10:25  wilms
; initial release into CVS
;
;
; -- PRE CVS follows: ---
; Version 1.0, Joern Wilms and Adrienne Allen
;    wilms@astro.uni-tuebingen.de
;    allenu@super.colorado.edu
; Version 1.1, JW, 2000/04/17
;    added feld, angr, mmc, and aneb options
;
;
;-

   IF (keyword_set(mmc)) THEN BEGIN 
       ab=[[  1, 12.00, 1.0,  1.], $
           [  2, 11.00, 1.0,  4.], $
           [  6,  8.65, 0.0, 12.], $
           [  7,  7.96, 0.0, 14.], $
           [  8,  8.87, 0.75, 16.], $
           [ 10,  8.14, 1.0, 20.], $
           [ 11,  6.32, 0.0, 23.], $
           [ 12,  7.60, 0.0, 24.], $
           [ 13,  6.49, 0.0, 27.], $
           [ 14,  7.57, 0.0, 28.], $
           [ 16,  7.28, 0.0, 32.], $
           [ 17,  5.28, 0.0, 35.], $
           [ 18,  6.58, 1.0, 40.], $
           [ 20,  6.35, 0.0, 40.], $
           [ 24,  5.69, 0.0, 52.], $
           [ 26,  7.52, 0.0, 56.], $
           [ 28,  6.26, 0.0, 59.]]
       ab[1,*]=(10.^ab[1,*])/1E12
       return,ab
   END 

   IF (keyword_set(feld)) THEN BEGIN 
       ab=[[  1, 1.00e+0, 1.0,  1.], $
           [  2, 9.77e-2, 1.0,  4.], $
           [  6, 3.98e-4, 1.0, 12.], $
           [  7, 1.00e-4, 1.0, 14.], $
           [  8, 8.51e-4, 1.0, 16.], $
           [ 10, 1.29e-4, 1.0, 20.], $
           [ 11, 2.14e-6, 1.0, 23.], $
           [ 12, 3.80e-5, 1.0, 24.], $
           [ 13, 2.95e-6, 1.0, 27.], $
           [ 14, 3.55e-5, 1.0, 28.], $
           [ 16, 1.62e-5, 1.0, 32.], $
           [ 17, 1.88e-7, 1.0, 35.], $
           [ 18, 4.47e-6, 1.0, 40.], $
           [ 20, 2.29e-6, 1.0, 40.], $
           [ 24, 4.84e-7, 1.0, 52.], $
           [ 26, 3.24e-5, 1.0, 56.], $
           [ 27, 8.60e-8, 1.0, 59.], $
           [ 28, 1.78e-6, 1.0, 59.]]
       return,ab
   END 
   IF (keyword_set(angr)) THEN BEGIN 
       ab=[[  1, 1.00e+0, 1.0,  1.], $
           [  2, 9.77e-2, 1.0,  4.], $
           [  6, 3.63e-4, 1.0, 12.], $
           [  7, 1.12e-4, 1.0, 14.], $
           [  8, 8.51e-4, 1.0, 16.], $
           [ 10, 1.23e-4, 1.0, 20.], $
           [ 11, 2.14e-6, 1.0, 23.], $
           [ 12, 3.80e-5, 1.0, 24.], $
           [ 13, 2.95e-6, 1.0, 27.], $
           [ 14, 3.55e-5, 1.0, 28.], $
           [ 16, 1.62e-5, 1.0, 32.], $
           [ 17, 1.88e-7, 1.0, 35.], $
           [ 18, 3.63e-6, 1.0, 40.], $
           [ 20, 2.29e-6, 1.0, 40.], $
           [ 24, 4.84e-7, 1.0, 52.], $
           [ 26, 4.68e-5, 1.0, 56.], $
           [ 27, 8.60e-8, 1.0, 59.], $
           [ 28, 1.78e-6, 1.0, 59.]]
       return,ab
   END    
   IF (keyword_set(aneb)) THEN BEGIN 
       ab=[[  1, 1.00e+0, 1.0,  1.], $
           [  2, 8.01e-2, 1.0,  4.], $
           [  6, 4.45e-4, 1.0, 12.], $
           [  7, 9.12e-5, 1.0, 14.], $
           [  8, 7.39e-4, 1.0, 16.], $
           [ 10, 1.38e-4, 1.0, 20.], $
           [ 11, 2.10e-6, 1.0, 23.], $
           [ 12, 3.95e-5, 1.0, 24.], $
           [ 13, 3.12e-6, 1.0, 27.], $
           [ 14, 3.68e-5, 1.0, 28.], $
           [ 16, 1.89e-5, 1.0, 32.], $
           [ 17, 1.93e-7, 1.0, 35.], $
           [ 18, 3.82e-6, 1.0, 40.], $
           [ 20, 2.25e-6, 1.0, 40.], $
           [ 24, 4.93e-7, 1.0, 52.], $
           [ 26, 3.31e-5, 1.0, 56.], $
           [ 27, 8.27e-8, 1.0, 59.], $
           [ 28, 1.81e-6, 1.0, 59.]]
       return,ab
   ENDIF    
   
   ;;
   ;; From here on the Wilms, Allen, McCray abundances
   ;;
   
   ;; Table as above, but abundances in the usual log version
   ab=[[ 1, 12.00, 1.0  ,  1.], $
       [ 2, 10.99, 1.0  ,  4.], $
       [ 6,  8.38, 0.5  , 12.], $
       [ 7,  7.88, 1.0  , 14.], $
       [ 8,  8.69, 0.6  , 16.], $
       [10,  7.94, 1.0  , 20.], $
       [11,  6.16, 0.25 , 23.], $
       [12,  7.40, 0.2  , 24.], $
       [13,  6.33, 0.02 , 27.], $
       [14,  7.27, 0.1  , 28.], $
       [15,  5.42, 0.6  , 31.], $
       [16,  7.09, 0.6  , 32.], $
       [17,  5.12, 0.5  , 35.], $
       [18,  6.41, 1.0  , 40.], $
       [20,  6.20, 0.003, 40.], $
       [22,  4.81, 0.002, 48.], $
       [24,  5.51, 0.03 , 52.], $
       [25,  5.34, 0.07 , 55.], $
       [26,  7.43, 0.3  , 56.], $
       [27,  4.92, 0.05 , 59.], $
       [28,  6.05, 0.04 , 59.]]
   
   ;; solar abundances
   IF (keyword_set(solar)) THEN BEGIN 
       ab[1,*]=[12.00,10.99,8.60,7.97,8.93,8.09,6.31,7.59, $
                6.48,7.55,5.57,7.27,5.27,6.56,6.34,4.93, $
                5.68,5.53,7.50,4.92,6.25]
   ENDIF 
   
   ab[1,*]=(10.^ab[1,*])/1E12
   
   return,ab
END 
