FUNCTION SPEX_EXTRACT, image, center_start, width, woff, width_sky,            $
                       pswitch = pswitch, filter = filter,                     $ 
                       specfit = specfit, step = step, degree = degree,        $
                       fitcoeff = fitcoeff, level = level,                     $
                       lev_halbwertsbreite = lev_halbwertsbreite,              $
                       nosky = nosky
;+
; NAME:          SPEX_EXTRACT
;
;
;
; PURPOSE:
;
;
;
; CATEGORY:      Astronomical Data Reduction: Longslit or Echelle Spectroscopy
;
;
;
; CALLING SEQUENCE:
;
;                flux=spex_extract(image,center_start,width,woff,width_sky     $
;                                  [ ,optionalin/output=optionalin/output      $
;                                                           ,/anykeyword ] )
;
;   
;
; INPUTS:
;
; image      input image                                            input
; center     central line of spectrum                               input
; width      width of spectrum                                      input
; woff       separation spectrum sky background                     input
; width_sky  width of sky background                                input
;
;
;
; OPTIONAL INPUTS:
;
; pswitch    plot switch: 0 no plots; 1 ,2 ,3  different            input
;                                     levels of communication     
;
; specfit    switch to fit gauss for central line of spectrum       input
; step       step witdh for fitting central line of spectrum        input
; degree     degree of polynom fit for central line of spectrum     input
; fitcoeff   fit coefficients of central line of spectrum           input/output
; filter     switch to filter cosmics                               input
; level      rejection level for cosmic filtering                   input
; lev_halbwertsbreite adjust rejection level far from center        input
;   
;
;   
; OUTPUTS:
;   
; flux       extracted flux                                               output
;
;
; OPTIONAL OUTPUTS:
;
; fitcoeff   fit coefficients of central line of spectrum           input/output
;
;
; KEYWORD PARAMETERS:
;
; nosky      prevent sky subtraction                                input
;            (desirable in case of calibration spectra)
;
;
; SIDE EFFECTS:
;
;
;
; RESTRICTIONS:
;
;
;
; EXAMPLE:
;
;
;
; MODIFICATION HISTORY:      author Stefan Dreizler       04/1999
;                            Stefan Dreizler, Sonja Schuh 02/2000
;
;-
;; ---------------------------------------------------------
;; --- DEFAULTS & DECLARATIONS
;;
   
;on_error,2
   
wtime=0.2
cdummy=''
s = size(image)
imagesave = image
   
;; --- set defaults
;;
siz = size(pswitch)
IF siz(1) EQ 0 THEN pswitch = 1
siz = size(specfit)
IF siz(1) EQ 0 THEN specfit = 1
siz = size(degree)
IF siz(1) EQ 0 THEN degree  = 3
siz = size(filter)
IF siz(1) EQ 0 THEN filter  = 0
siz = size(level)
IF siz(1) EQ 0 THEN level   = 1.2 ELSE levelsave = level
lev_halbwertsbreite = 5.
siz = size(step)
IF siz(1) EQ 0 THEN step    = 1

;; --- declare arrays
;;
x    = fltarr(2*width+1)        ; image section perpendicular to dispersion
y    = fltarr(2*width+1)        ; flux in section x
cen  = fltarr(s(1))             ; centers of spectrum along dispersion
bound= fltarr(2,4)              ; bounds for gauss fits
xaxis= findgen(s(1))            ; array with pixels in dispersion
ima  = fltarr(s(1),width+1 + 2*width_sky+2 +2) ; cleaned  extraction area
imas = fltarr(s(1),width+1 + 2*width_sky+2 +2) ; original extraction area 
sky1 = fltarr(width_sky+1)      ; area of sky background one  side
sky2 = fltarr(width_sky+1)      ; area of sky background other side
sky  = fltarr(s(1))             ; flux in the sky region
edge = fltarr(s(1),6)           ; area between spectrum and sky
flux = fltarr(s(1))             ; extracted flux

;; ---------------------------------------------------------
;; --- FIND SPECTRUM ON THE CHIP  
;;
;; --- fit gauss to determine centers
IF (specfit EQ 1) THEN BEGIN   
    
    FOR ii=0,s(1)-1,step DO BEGIN         ;; ====== FIRST LOOP ALONG DISPERSION 
        siz = size(fitcoeff)                      
        center = float(center_start)                ;; center at left edge
        
        FOR k = 1,siz(1) DO      $
          center = center + fitcoeff(k)*float(ii)^k ;; account for slopes
        
        FOR j=0,2*width DO BEGIN                    ;; cut of width "width+1"   
            x(j) = fix(center+0.5) - width + j    ;; perpendicular to dispersion
            y(j) = image(ii,x(j))                      
        ENDFOR 
        
        yfit = gaussfit(x,y,a,nterms=4)             ;; use IDL gaussfit 
        cen(ii) = a(1)                             
        
        IF (pswitch GE 3) THEN BEGIN                ;; plot fit
            plot,x,y,xtitle='pixel position',psym=1,ytitle='counts',           $
              title='Gauss Fit'
            oplot,x,a(3)+a(0)*exp(-((x-a(1))/a(2))^2/2.),color=20
            xyouts,x(width-1),y(width)/2,ii
            wait,wtime
        ENDIF
        
    ENDFOR                                ;; ====== END FIRST LOOP ======
    
;; ---------------------------------------------------------
;; --- NOW FIT POLYNOMIAL TO THE CENTRAL PIXEL OF SPECTRUM
;;
    ind = where (cen NE 0)
    ;; --- display result of gauss fits along dispersion
    IF (pswitch GE 1) THEN BEGIN                    
        newplot:
        ;; --- polynomial fit
        fitcoeff = poly_fit(xaxis(ind),cen(ind),degree,/double) 
        cenfit = fitcoeff(0)
        FOR k = 1,degree DO cenfit = cenfit + fitcoeff(k)*xaxis^k
        
        plot,xaxis(ind),cen(ind),ystyle=1,                                     $
          xtitle='pixel position direction dispersion',ytitle='pixel position',$
          title ='pixel position of the spectrum'
        oplot,xaxis,cenfit,color=20
        xyouts,xaxis(1),min(cenfit)+.5*(max(cenfit)-min(cenfit)),              $
          'polynom fit of degree '+strtrim(string(degree),2),color=20
        PRINT,' '
        PRINT,'% SPEX_EXTRACT: ready to continue?      press return'
        PRINT,'% SPEX_EXTRACT: reject points from fit? press r'
        PRINT,'% SPEX_EXTRACT: new polynomial fit?     enter degree'
        
        read,cdummy                                 ;; wait for orders
        CASE cdummy OF
            
            '' : PRINT,'% SPEX_EXTRACT: continuing '            
            'r': BEGIN             
                ;; --- in case points are to be rejected from fit
                PRINT,' '
                message,'Left   mouse click   : reject points',/inf
                message,'Middle mouse click   : end ',/inf
                message,'Right  mouse click   : reject area ',/inf
                
                REPEAT BEGIN
                    
                    plot,xaxis(ind),cen(ind),ystyle=1,                         $
                      xtitle='pixel position direction dispersion',            $
                      ytitle='pixel position',                                 $
                      title ='pixel position of the spectrum'
                    cursor,xx,yy,/data
                    mouse=!mouse.button
                    CASE mouse OF
                        
                        1: BEGIN 
                            diff = ((xaxis-xx)/xx)^2 + ((cen-yy)/yy)^2
                            x_reject = where (diff EQ min(diff))
                            ind_n = where(ind NE x_reject(0))
                            ind = ind(ind_n)
                            wait,0.3
                        END
                        
                        2: PRINT,'% SPEX_EXTRACT: finished rejection'
                        4: BEGIN
                            PRINT,' '
                            message,'next point to mark area ...',/inf
                            wait,0.3
                            cursor,xx2,yy,/data
                            wait,0.3
                            message,'area marked ...',/inf
                            xx3 = [xx,xx2]
                            ind3 = sort(xx3)
                            ind2 = where(                    $
                               xaxis(ind) LT xx3(ind3(0)) OR $
                               xaxis(ind) GT xx3(ind3(1))    )
                            ind = ind(ind2)
                            wait,0.3
                        END
                    ENDCASE 
                ENDREP UNTIL (mouse eq 2)
                
                GOTO,newplot                        ;; re-display 
            END                                     ;; end rejection
            ELSE: BEGIN
                degree = fix(cdummy)
                GOTO,newplot
            END    
        ENDCASE
        
    ENDIF                                           ;; end display/fit centers    
    cen = cenfit                                    ;; save fitted centers
    
ENDIF ELSE BEGIN                                    ;; no fit
    
    siz = size(fitcoeff)
    ;; --- position of spectrum left edge
    cen = fltarr(s(1)) + float(center_start)     
    ;; --- account for slope
    FOR k = 1,siz(1) DO cen = cen + fitcoeff(k)*findgen(s(1))^k 
    
ENDELSE

;; ---------------------------------------------------------
;; --- FILTER COSMICS
;;

IF filter EQ 1 THEN BEGIN
    
    FOR ii=0,s(1)-1 DO BEGIN              ;; ====== SECOND LOOP ALONG DISPERSION 
        redo:
        FOR j=0,2*width DO BEGIN
            x(j) = fix(cen(ii)+.5) - width + j
            y(j) = image(ii,x(j))
        ENDFOR 
        
        ;; --- use IDL gaussfit for first fit
        IF ii EQ 0 THEN yfit = gaussfit(x,y,a,nterms=4) 
        bound(0,1) = cen(ii)
        bound(1,1) = cen(ii)
        bound(0,0) = 0.9*a(0)
        bound(1,0) = 1.1*a(0)
        bound(0,2) = 0.9*a(2)
        bound(1,2) = 1.1*a(2)
        bound(0,3) = 0.1*a(3)
        bound(1,3) = 10.*a(3)
        asave = a 
        bsave = bound                               ;; keep fit 
        
        ;; --- use own gaussfit for other fits
        IF ii GE 0 THEN BEGIN   
            result = tripp_curvefit ( x,y,[replicate(1.0,n_elements(x))],      $
                                      a,sigmaa,                                $
                                      FUNCTION_NAME ='fit_gauss' ,             $
                                      itmax=150, tol=1.e-4,bounds=bound,       $
                                      /noderivative)
            IF (result(0) EQ -1) THEN BEGIN
                a = asave
                bound = bsave
                GOTO, weiter
            ENDIF 
            IF n_elements(a) LT 4 THEN BEGIN
                a = asave
                bound = bsave
                GOTO,weiter
            ENDIF 
            yfit = a(0)*exp(-0.5*((x-a(1))/a(2))^2) + a(3)
        ENDIF
        
        ;; --- determine rejected pixel
        levadd = (level - 1.) *exp(-0.5*((x-a(1))/(a(2)*lev_halbwertsbreite))^2)  
        ind = where (y/a(0)+1000. GT levadd+(yfit/a(0)+1000.),count) 
        ;; --- display results
        IF (pswitch GE 2) THEN BEGIN
            plot,x,y,xtitle='pixel position',psym=1,ytitle='counts',           $
              title='Gauss Fit with rejection above'+strtrim(string(level),2)
            oplot,x,yfit,color=20
            IF count GT 0 THEN $
              oplot,[x(ind),x(ind)],[y(ind),y(ind)],color=100,psym=1
            xyouts,x(width-1),y(width)/2,ii
            wait,wtime
        ENDIF
        
        ;; --- case rejected pixel exist
        IF count GT 0 OR ii EQ 0 THEN BEGIN         ;; ii=0 hinzugefuegt 
            idx = where (y/a(0)+1000. LE levadd+(yfit/a(0)+1000.),count)
            IF count LT 5 THEN GOTO,weiter
            aa = asave                              ;;reuse older fit
            bound(0,1) = cen(ii)                    ;;now fix center of gaussfit
            bound(1,1) = cen(ii)            ;;to prevent irritation from cosmics
            bound(0,0) = 0.001*asave(0)
            bound(1,0) = 1000.*asave(0)
            bound(0,2) = 0.001*asave(2)
            bound(1,2) = 1000.*asave(2)
            bound(0,3) = 0.001*asave(3)
            bound(1,3) = 1000.*asave(3)
            result = tripp_curvefit(x(idx),y(idx),[replicate(1.,n_elements(x))]$
                                    ,aa,sigmaa, FUNCTION_NAME='fit_gauss',     $
                                    itmax=150,tol=1.e-4,bounds=bound,          $
                                    /noderivative)
            IF n_elements(aa) LT 4 THEN BEGIN
                a = asave
                bound = bsave
                GOTO,weiter
            ENDIF 
            yfit = aa(0)*exp(-0.5*((x-aa(1))/aa(2))^2) + aa(3)
            ind = where (y/aa(0)+1000. GT levadd+(yfit/aa(0)+1000.),count)
            a = aa                    ;; keep fit as better guess for next pixel
            
            IF (pswitch GE 1 AND (count GT 0 OR ii EQ 0)) THEN BEGIN ;; ii=0 hinzugefuegt 
                ;; --- display results and allow interaction
                re_plot:
                plot,x,y,xtitle='pixel position',psym=1,ytitle='counts',title= $
                  'Gauss Fit with rejection above '+strtrim(string(level),2)
                oplot,x,yfit,color=150
                IF count GT 0 THEN $
                  oplot, [x(ind),x(ind)], [y(ind),y(ind)], color=100, psym=1
                xyouts, x(width-1), y(width)/2, ii
                oplot,[fix(cen(ii)+.5)-width/2,fix(cen(ii)+.5)-width/2],[0.,y(width)],color=150,linestyle=1
                oplot,[fix(cen(ii)+.5)+width/2,fix(cen(ii)+.5)+width/2],[0.,y(width)],color=150,linestyle=1
                typeagain:
                PRINT,' '
                PRINT,'% SPEX_EXTRACT: accepted rejection? press return'
                PRINT,'% SPEX_EXTRACT: modify manually   ? press m'
                PRINT,'% SPEX_EXTRACT: keep original data? press k'
                PRINT,'% SPEX_EXTRACT: new plot switch   ? press p'
                PRINT,'% SPEX_EXTRACT: new level         ? enter level'
                IF count EQ 0 AND ii EQ 0 THEN BEGIN
                    PRINT, '% SPEX_EXTRACT: '$
                      +'No rejection necessary; unless you want to modify '
                    PRINT, '%               the level or plot switch, press '  $
                      +'return to continue'
                ENDIF
                read,cdummy                         ;; wait for orders
                PRINT,' '
                CASE cdummy OF
                    '' : PRINT,'% SPEX_EXTRACT: continuing'
                    'p': BEGIN    
                        pswitch_save = pswitch
                        PRINT,' '
                        PRINT,'% SPEX_EXTRACT: enter new plot switch (current '$
                          +strtrim(string(pswitch),2)+' )'
                        PRINT,'%               0    for silent filtering '
                        PRINT,'%               2, 3 for plots at each pixel'
                        pswitchread=''
                        read,pswitchread
                        pswitch=fix(pswitchread)
                        IF pswitch EQ 0 THEN BEGIN
                            PRINT,' '
                            PRINT,'% SPEX_EXTRACT: Please confirm new plot'  + $
                              ' switch of 0 : press return'
                            PRINT,'% SPEX_EXTRACT:              for a second'+ $
                              ' chance    : any other letter'
                            read,cdummy
                            IF cdummy NE '' THEN GOTO, typeagain
                        ENDIF
                    END
                    'k': BEGIN    
                        count = 0                   ;; keep original data
                        a = asave
                        bound = bsave
                    END
                    'm': BEGIN    
                        count = 0
                        ind = indgen(n_elements(x))
                        idx = indgen(n_elements(x))
                        
                        PRINT,' '
                        message,'Left   mouse click   : reject points',/inf
                        message,'Middle mouse click   : end ',/inf
                        
                        REPEAT BEGIN
                            result=tripp_curvefit(x(idx),y(idx),$
                                                  [replicate(1.,n_elements(x))]$
                                                  ,aa,sigmaa,                  $
                                                  FUNCTION_NAME='fit_gauss',   $
                                                  itmax=150,tol=1.e-4,         $
                                                  /noderivative)
                            plot, x, y, psym=1, $
                              xtitle='pixel position',ytitle='counts'
                            IF n_elements(aa) EQ 4 THEN BEGIN
                                yfit = $
                                  aa(0)*exp(-0.5*((x-aa(1))/aa(2))^2) + aa(3)
                                oplot,x,yfit,color=150
                            ENDIF ELSE BEGIN
                                PRINT,' '
                                PRINT,'% SPEX_EXTRACT: '+ $
                                  'try fitting again ? press return'
                                PRINT,'% SPEX_EXTRACT: '+ $
                                  'abort fitting     ? press a'
                                read,cdummy         ;; wait for orders
                                IF cdummy NE 'a' THEN BEGIN 
                                    a = asave
                                    bound = bsave
                                    GOTO, weiter 
                                ENDIF 
                            ENDELSE
                            IF count GT 0 THEN oplot, $
                              [x(ind),x(ind)],[y(ind),y(ind)],color=100,psym=1
                            xyouts, x(width-1), y(width)/2, ii
                            cursor,xx,yy,/data
                            mouse=!mouse.button
                            CASE mouse OF
                                
                                1: BEGIN 
                                    diff = ((x-xx)/xx)^2 + ((y-yy)/yy)^2
                                    x_reject = where(diff EQ min(diff))
                                    ind_n = where(idx EQ x_reject(0),howoften)
                                    IF howoften NE 0 THEN BEGIN  
                                        IF count EQ 0 THEN ind   = idx(ind_n)
                                        IF count GT 0 THEN ind   = [ind,idx(ind_n)]
                                        idx_n = where(idx NE x_reject(0))
                                        idx   = idx(idx_n)
                                        count = count + 1
                                        wait,0.3
                                    ENDIF 
                                END
                                
                                2: BEGIN
                                    PRINT,'% SPEX_EXTRACT: finished rejection'
                                    GOTO,re_plot
                                END
                                
                                ELSE: PRINT, $
                                  '% SPEX_EXTRACT: right cursor not allowed'
                                
                            ENDCASE 
                        ENDREP UNTIL (mouse eq 2)
                        a = aa
                    END
                    ELSE: BEGIN
                        level = float(cdummy)
                        IF level EQ 0. THEN GOTO, typeagain
                        GOTO,redo
                    END
                ENDCASE
            ENDIF
            
            ;; --- apply cleaning to image
            IF count GT 0 THEN FOR kk = 0,n_elements(ind)-1 DO BEGIN 
                image(ii,x(ind(kk))) = yfit(ind(kk)) 
                y(ind(kk)) =  yfit(ind(kk))
            ENDFOR
            
            weiter:             
            ;; --- end interaction for this dispersion pixel range
        ENDIF                   ;; end case rejected pixel exist
                
    ENDFOR                                ;; ====== END SECOND LOOP ======
    IF EXIST(pswitch_save) THEN pswitch = pswitch_save
ENDIF                                               ;; end filtering

;; ---------------------------------------------------------
;; --- DO FLUX EXTRACTION NOW !
;;   

FOR ii=0,s(1)-1 DO BEGIN                  ;; ====== THIRD LOOP ALONG DISPERSION 
    
    ;; --- spectrum
    flux(ii) = total(image(ii,fix(cen(ii)+.5)-width/2:fix(cen(ii)+.5)+width/2)) 
    ;; --- sky ranges
    width_min = max([fix(cen(ii)+.5)-woff-width_sky,0])
    width_sky1 = fix(cen(ii)+.5) - woff - width_min
    sky1 = image(ii,width_min:fix(cen(ii)+.5)-woff) 
    width_max = min([fix(cen(ii)+.5)+woff+width_sky,s(2)-1])
    width_sky2 = width_max - fix(cen(ii)+.5) - woff
    sky2 = image(ii,fix(cen(ii)+.5)+woff:width_max)
    ;; --- median filter for sky
    sky(ii) = float(width+1)*(median(sky1)+median(sky2))/2. 
    
    ;; --- sky subtraction
    IF NOT keyword_set(nosky) THEN flux(ii) = flux(ii) - sky(ii) 
    
    ;; --- boundaries between spectrum and sky fields
    edge(ii,0) =  fix(cen(ii)+.5) - width/2
    edge(ii,1) =  fix(cen(ii)+.5) + width/2
    edge(ii,2) =  width_min
    edge(ii,3) =  fix(cen(ii)+.5) -woff
    edge(ii,4) =  fix(cen(ii)+.5) +woff 
    edge(ii,5) =  width_max
    
    ;; --- fill images for 3D plots ima: filtered, imas: original
    imas(ii,width_sky-width_sky1:width_sky)                           =  $
      imagesave(ii,fix(edge(ii,2)):fix(edge(ii,3)))
    imas(ii,width_sky+1)                           = -100.
    imas(ii,width_sky+2:width_sky+width+2)         =  $
      imagesave(ii,fix(edge(ii,0)):fix(edge(ii,1)))
    imas(ii,width_sky+width+3)                     = -100.
    imas(ii,width_sky+width+4:width_sky+width+4+width_sky2) =  $
      imagesave(ii,fix(edge(ii,4)):fix(edge(ii,5)))
    image(ii,width_min:fix(cen(ii)+.5)-woff) = median(sky1)
    image(ii,fix(cen(ii)+.5)+woff:width_max) = median(sky2)
    ima(ii,width_sky-width_sky1:width_sky)                            =  $
      image(ii,fix(edge(ii,2)):fix(edge(ii,3)))
    ima(ii,width_sky+1)                            = -100.
    ima(ii,width_sky+2:width_sky+width+2)          =  $
      image(ii,fix(edge(ii,0)):fix(edge(ii,1)))
    ima(ii,width_sky+width+3)                      = -100.
    ima(ii,width_sky+width+4:width_sky+width+4+width_sky2)  =  $
      image(ii,fix(edge(ii,4)):fix(edge(ii,5)))
    
ENDFOR                                    ;; ====== END THIRD LOOP ======

;; ---------------------------------------------------------
;; --- DISPLAY RESULTS OF FILTERING (IF DONE) AND EXTRACTION 
;;   
IF (pswitch GE 1) THEN BEGIN
    loadct,3 
    ;; --- display clean minus original image
    IF filter EQ 1 THEN BEGIN   
        shade_surf,imas-ima,xtitle='pixel direction dispersion',ytitle='pixel',$
          ztitle='counts',title='rejected cosmics'
        PRINT,' '
        PRINT,'% SPEX_EXTRACT: ready to continue? press return'
        read,cdummy
    ENDIF
    ;; --- display extraction ranges
    shade_surf,ima,xtitle='pixel direction dispersion',ytitle='pixel',         $ 
      ztitle='counts',title='extraction range' 
    PRINT,'% SPEX_EXTRACT: ready to continue? press return'
    read,cdummy
    
    loadct,39
    
;; ---------------------------------------------------------
;; --- MODIFY RESULTS IF DESIRED 
;;   
    
replot:
    plot,flux,xtitle='pixel',ytitle='counts',title='extracted spectrum'
    PRINT,' '
    PRINT,'% SPEX_EXTRACT: ready to continue    ? press return'
    PRINT,'% SPEX_EXTRACT: modify manually      ? press m'
    PRINT,'% SPEX_EXTRACT: linear interpolation ? press l'
    read,cdummy
    CASE cdummy OF 
        'l': BEGIN
            PRINT,' '
            message,'Left   mouse click   : mark region to be modified',/inf
            cursor,xx,yy,/data
            wait,0.3
            ipix = fix(xx)
            left_edge = min([20,ipix])
            right_edge= min([20,n_elements(flux)-1-ipix])
            xpix = ipix+indgen(left_edge+right_edge+1)-left_edge
            plot,xpix,flux(ipix-left_edge:ipix+right_edge),xtitle='pixel',ytitle='counts',    $
              title='region to be modified',ystyle=1
            PRINT,' '
            message,'mark left point for linear interpolation',/inf
            cursor,xx1,yy1,/data
            ipix1 = fix(xx1+0.5)
            wait,0.3
            PRINT,' '
            message,'mark right point for linear interpolation',/inf
            cursor,xx2,yy2,/data
            ipix2 = fix(xx2+0.5)
            wait,0.3
            xaxis = indgen(n_elements(flux))
            xpix = where(xaxis GT ipix1 AND xaxis LT ipix2)
            IF ipix1-2 GE 0 THEN flux_l = mean(flux(ipix1-2:ipix1)) $
            ELSE flux_l = flux(ipix1)
            IF ipix2+2 LE n_elements(flux)-1 THEN $
              flux_r = mean(flux(ipix2:ipix2+2)) $
            ELSE flux_r = flux(ipix2)
            plots,ipix1,flux(ipix1),color=20
            FOR kk = 0,n_elements(xpix)-1 DO BEGIN
                flux(xpix(kk)) = $
                  interpol ([flux_l,flux_r],[ipix1,ipix2],xpix(kk))
                plots,xpix(kk),flux(xpix(kk)),/continue,color=20
            ENDFOR 
            PRINT,' '
            PRINT,'% SPEX_EXTRACT: ready to continue? press return'
            read,cdummy
            GOTO, replot
        END        
        'm': BEGIN
            PRINT,' '
            message,'Left   mouse click   : mark region to be modified',/inf
            cursor,xx,yy,/data
            wait,0.3
            ipix = fix(xx)
            xpix = ipix+indgen(31)-15
            plot,xpix,flux(ipix-15:ipix+15),xtitle='pixel',ytitle='counts',    $
              title='region to be modified',ystyle=1
            PRINT,' '
            message,'Left   mouse click   : mark position to be modified',/inf
            cursor,xx,yy,/data
            wait,0.3
            diff = ((xpix-xx)/xx)^2 + ((flux(ipix-15:ipix+15)-yy)/yy)^2
            ii   = xpix (where (diff EQ min(diff)))
            FOR j=0,2*width DO BEGIN
                x(j) = fix(cen(ii)+.5) - width + j
                y(j) = image(ii,x(j))
            ENDFOR 
            
            ind = indgen(n_elements(x))
            idx = indgen(n_elements(x))
            count = 0
            
            PRINT,' '
            message,'Left   mouse click   : reject points',/inf
            message,'Middle mouse click   : end ',/inf
            
            REPEAT BEGIN
                refit:
                result = gaussfit(x(idx),y(idx),aa,nterms=4)
                result = tripp_curvefit(x(idx),y(idx),                         $
                                        [replicate(1.,n_elements(x))],aa,      $
                                        sigmaa,FUNCTION_NAME='fit_gauss',      $
                                        itmax=150,tol=1.e-4, /noderivative)   
                plot,x,y,xtitle='pixel position',psym=1,ytitle='counts'
                IF n_elements(aa) eq 4 THEN BEGIN
                    yfit = aa(0)*exp(-0.5*((x-aa(1))/aa(2))^2) + aa(3)
                    oplot,x,yfit,color=150
                ENDIF ELSE BEGIN
;                    PRINT,' '
;                    PRINT,'% SPEX_EXTRACT: try fitting again ? press return'
;                    PRINT,'% SPEX_EXTRACT: abort fitting     ? press a'
;                    read,cdummy                     ;; wait for orders
;                    IF cdummy EQ 'a' THEN BEGIN 
                        PRINT,'% SPEX_EXTRACT: '+ $
                          'Fitting failed, try linear interpolation instead ...'
                        GOTO, replot
;                    ENDIF ELSE GOTO, refit
                ENDELSE
                IF count GT 0 THEN $
                  oplot, [x(ind),x(ind)], [y(ind),y(ind)], color=100, psym=1
                xyouts,x(width-1),y(width)/2,ii
            PRINT,' '
            message,'Left   mouse click   : reject points',/inf
            message,'Middle mouse click   : end ',/inf
                cursor,xx,yy,/data
                mouse=!mouse.button
                CASE mouse OF
                    
                    1: BEGIN 
                        diff = ((x-xx)/xx)^2 + ((y-yy)/yy)^2
                        x_reject = where (diff EQ min(diff))
                        ind_n = where(idx EQ x_reject(0))
                        IF count EQ 0 THEN ind   = idx(ind_n)
                        IF count GT 0 THEN ind   = [ind,idx(ind_n)]
                        idx_n = where(idx NE x_reject(0))
                        idx   = idx(idx_n)
                        count = count + 1
                        wait,0.3
                    END
                    
                    2: PRINT,'% SPEX_EXTRACT: finished rejection'
                    
                    ELSE: PRINT,'% SPEX_EXTRACT: right cursor not allowed'
                    
                ENDCASE 
                
            ENDREP UNTIL (mouse eq 2)
            
            ;; --- clean image
            IF count GT 0 THEN FOR kk = 0,n_elements(ind)-1 DO BEGIN 
                image(ii(0),x(ind(kk))) = yfit(ind(kk)) 
                y(ind(kk)) =  yfit(ind(kk))
            ENDFOR
            ;; --- spectrum
            flux(ii(0)) = total(image(ii(0), fix(cen(ii(0))+.5)-width/2        $
                                      :      fix(cen(ii(0))+.5)+width/2)) 
            ;; --- sky subtraction
            IF NOT keyword_set(nosky) THEN flux(ii(0)) = flux(ii(0))-sky(ii(0)) 
            GOTO, replot
        END        
        ELSE : PRINT,'% SPEX_EXTRACT: continuing'
    ENDCASE
ENDIF


;; ---------------------------------------------------------
;; --- END
;;   

;; --- reset some values
loadct,39
siz = size(levelsave)
IF siz(1) EQ 0 THEN level   =  levelsave

;; --- end
PRINT,' '
PRINT,'% SPEX_EXTRACT: Extraction done.'

return, flux
END 
