;@tripp_exist
@tripp_curvefit
@spex_extract
@spex_fit_gauss
@spex_fit_wave
@spex_fluxcal
PRO SPEX_REDUCE, logFile, pswitch=pswitch, filter=filter, level=level,     $
                 lev_halbwertsbreite=lev_halbwertsbreite,                  $
                 useoldwave=useoldwave,spectrum=spectrum,width=width,      $
                 start=start,stdfluxtable=stdfluxtable,                    $
                 useoldstandard=useoldstandard,fluxcalibonly=fluxcalibonly,$
                 signed=signed,overscan=overscan,center=center,            $
                 rotate=rotate,wave_fit=wave_fit,xr=xr
;+
; NAME:               SPEX_REDUCE
;
;
;
; PURPOSE:            Extract spectrum(spectra) from CCD-frame(s):
;                     Apply bias + flatfield corrections
;                     Make wavelength calibration
;                     Make flux calibration if desired
;
;
; CATEGORY:           Astronomical Data Reduction: Longslit or Echelle Spectroscopy
;
;
;
; CALLING SEQUENCE:   SPEX_REDUCE, logFile [,/anykeyword]
;
;
;
; INPUTS:             logFile         (see below for format restrictions);
;                     FITS data files (via logFile only): 
;                          OBJECT   with its BIAS, FLAT, WAVE
;                          STANDARD with its BIAS, FLAT, WAVE
;                     wavelength data file (via spex_fit_wave, e.g. HeAr.dat) 
;                     flux calibration data file          (e.g. fg191b2b.dat)
;                          mandatory only if flux calibration desired and
;                          keyword /useoldstandard NOT set 
;       
;
; OPTIONAL INPUTS:    stdfluxtable=stdfluxtable           (e.g. fg191b2b.dat)
;                     pswitch=pswitch : communicated to spex_extract
;                     filter =filter  : communicated to spex_extract 
;                     level  =level   : communicated to spex_extract
;                     lev_halbwertsbreite=lev_halbwertsbreite : "
;
;
;   
; KEYWORD PARAMETERS: 
;                     /fluxcalibonly  : skips Reduction & Extraction 
;                                       (assumed to be done already)
;                                        - once for all objects - !   
;                     /useoldstandard : skips large parts of i=0 Loop
;                                       (assumed to be done already)   
;                                        - for first object(=std) only!   
;                     /useoldwave     : skips re-determination of
;                                       wavelength fit coefficients
;                                       (assumed to be done already)   
;                                        - once for all objects - !   
;
;
; OUTPUTS:            wavelength calibrated spectrum as x,y table:
;                                       log.object_spec
;                     flux       calibrated spectrum as x,y table:
;                                       log.object_fcspec
;
;
;   
; INTERNAL OUTPUTS:                     log.object_wave,                 
;                     selected wavelengths data file: e.g.
;                                       HeAr.dat_select
;
;
; COMMON BLOCKS:      none
;
;
;
; SIDE EFFECTS:
;
;
;
; RESTRICTIONS:
;
;
;
; FUNCTIONS:         spex_extract  (uses tripp_curvefit from tripp_package)
;                    spex_fit_wave
;                    spex_fluxcal   
;                    exist (ccd_package)     or tripp_exist (tripp_package)
;
; PROCEDURES:
;
;
;
; EXAMPLE:           spex_reduce,'objects.log'
;
;
;
; MODIFICATION HISTORY:    written in  January 2000  by 
;                          Stefan Dreizler, Sonja Schuh 
;                          signed integer, overscan bias added S.D.
;                          
;-
;; ---------------------------------------------------------
;; --- DEFAULTS
;;   
   loadct,39
   cdummy = ''
   IF NOT EXIST(wave_fit)THEN wave_fit = 'wave_fit.dat'
   IF NOT EXIST(spectrum)THEN spectrum = 'HeAr.dat'
   IF NOT EXIST(width)   THEN width    = 14 
   IF NOT EXIST(start)   THEN start    = 1 
   IF NOT EXIST(pswitch) THEN pswitch = 1
   IF NOT EXIST(filter)  THEN filter  = 1
   IF NOT EXIST(level)   THEN level   = 1.1
   IF NOT EXIST(lev_halbwertsbreite) THEN lev_halbwertsbreite = 5.
   IF KEYWORD_SET(fluxcalibonly)           $
     AND NOT KEYWORD_SET(useoldstandard)   $
     AND NOT       EXIST(stdfluxtable)     $
     THEN BEGIN 
       PRINT,'% SPEC_REDUCE: WARNING This keyword combination means nothing is done at all:' 
       PRINT,'% SPEC_REDUCE:         fluxcalib without stdfluxtable or useoldstandard ... !'
       PRINT,'% SPEC_REDUCE:         meaning: Flux calibration will not start unless you additionally'
       PRINT,'% SPEC_REDUCE:         specify a  stdfluxtable = ... or set  /useoldstandard '
   ENDIF
   IF KEYWORD_SET(useoldstandard) AND EXIST(stdfluxtable) THEN BEGIN
       PRINT,'% SPEC_REDUCE: WARNING stdfluxtable will be ignored as /useoldstandard is set,'
       PRINT,'% SPEC_REDUCE:         the input of stdfluxtable is obsolete, you may skip it.'
   ENDIF  
   IF KEYWORD_SET(fluxcalibonly) THEN BEGIN
       PRINT,'% SPEC_REDUCE:         Are you sure that all the wavelength calibrated spectra '
       PRINT,'% SPEC_REDUCE:         do exist ? If not, you will be in trouble in a second.  '
   ENDIF
   IF KEYWORD_SET(useoldwave) THEN BEGIN
       PRINT,'% SPEC_REDUCE:         Are you sure that all the wavelength calibrated spectra '
       PRINT,'% SPEC_REDUCE:         do exist ? If not, you will be in trouble in a second.  '
   ENDIF
   IF KEYWORD_SET(useoldstandard) THEN BEGIN
       PRINT,'% SPEC_REDUCE:         Are you sure that the first flux calibrated spectrum '
       PRINT,'% SPEC_REDUCE:         does exist ? If not, you will be in trouble in a second.  '
   ENDIF
   skipstandard = 0
     
;; ---------------------------------------------------------
;; --- ABOUT FORMAT OF LOG FILE
;;   
;; Log File has to be of following form:
;; First entry is number of object structure information blocks
;;   to follow
;; Individual entries must respect information block structure
;; First block must contain standard star: will be used for
;;   flux calibration !  
   
;; ---------------------------------------------------------
;; --- DEFINE STRUCTURE 
;;   
   log = { SPEX_LOG, object  : '', $
           bias              : '', $
           flat              : '', $
           wave              : '', $
           wavedegree        : 0 , $
           cuts_through_spec : 0 , $
           usable_begin      : 0 , $
           usable_end        : 0 , $
           specwidth         : 0 , $
           skyoff            : 0 , $
           skywidth          : 0   }
   
;; ---------------------------------------------------------
;; --- GET ESSENTIAL INFORMATION
;;   
   GET_LUN,unit
   OPENR,unit,logFile
   READF,unit,nr_objects
   FREE_LUN,unit
   PRINT,'% SPEX_REDUCE: Starting reduction & extraction of ' $
     +STRING(FIX(nr_objects))+' objects'
   
;; ======================== BIG LOOP
   FOR i=0,nr_objects-1 DO BEGIN
       
;; ---------------------------------------------------------
;; --- READ IN STRUCTURE(S)
;;   
       GET_LUN,unit
       OPENR,unit,logFile
       READF,unit,dummy
       
       FOR n=0,i DO BEGIN       ;(read in Structure - for actual object!)
           READF, unit, log     ;log immer wieder ueberschreiben!
       ENDFOR
       
       FREE_LUN,unit
       
       ;; --- ensure correct usage of read-in log.things
       
       log.object            = STRTRIM(log.object,2)          
       log.bias              = STRTRIM(log.bias  ,2)  
       log.flat              = STRTRIM(log.flat  ,2) 
       log.wave              = STRTRIM(log.wave  ,2)   
       log.wavedegree        = FIX    (log.wavedegree) 
       log.cuts_through_spec = FIX    (log.cuts_through_spec) 
       log.usable_begin      = FIX    (log.usable_begin) 
       log.usable_end        = FIX    (log.usable_end) 
       log.specwidth         = FIX    (log.specwidth) 
       log.skyoff            = FIX    (log.skyoff)  
       log.skywidth          = FIX    (log.skywidth) 

       IF log.specwidth MOD 2 NE 0 THEN log.specwidth = log.specwidth + 1

       ;; --- don't do things that already have been done 
       IF KEYWORD_SET(fluxcalibonly) THEN GOTO, fluxcalibstart
       
;; ---------------------------------------------------------
;; --- READ IN STANDARD, BIAS, FLAT  IF NECCESSARY
;;   
       IF i EQ 0 THEN BEGIN
           STANDARD = readfits(log.object,hobject)
           IF (keyword_set(rotate)) THEN STANDARD = rotate(STANDARD,1)
           IF ( n_elements(signed)   EQ 1 ) THEN SPEX_SIGNED,STANDARD
           IF ( n_elements(overscan) EQ 0 ) THEN BEGIN 
               BIAS     = readfits(log.bias  ,h) ;each produced with tripp_zero/flat
               IF (keyword_set(rotate)) THEN BIAS = rotate(BIAS,1)
           ENDIF 
           FLAT     = readfits(log.flat  ,h)                                       ;bias has already been subtracted
           IF (keyword_set(rotate)) THEN FLAT = rotate(FLAT,1)
           dimFlat = size(FLAT)
           illum = dblarr(dimFlat(1))
           FOR kk = 0,dimFlat(1)-1 DO BEGIN
               illum(kk) = total(FLAT(kk,*))/float(dimFlat(2))
               FLAT(kk,*)= FLAT(kk,*)/illum(kk)             ;;just to make sure ...
           ENDFOR
;;;;;           FLAT     = FLAT/median(FLAT)
           WAVE     = readfits(log.wave  ,h)
           IF (keyword_set(rotate)) THEN WAVE = rotate(WAVE,1)
           IF ( n_elements(signed)  EQ 1) THEN SPEX_SIGNED,WAVE
           testbias = log.bias
           testflat = log.flat
       ENDIF
       IF log.bias NE testbias THEN BEGIN
           IF ( n_elements(overscan) EQ 0 ) THEN BEGIN 
               BIAS     = readfits(log.bias  ,h)
               IF (keyword_set(rotate)) THEN BIAS = rotate(BIAS,1)
           ENDIF 
           PRINT,'% SPEX_REDUCE: Using '+log.bias+' for '+log.object 
       ENDIF ELSE PRINT,'% SPEX_REDUCE: Using '+log.bias+' for '+log.object
       IF log.flat NE testflat THEN BEGIN
           FLAT     = readfits(log.flat  ,h)
           IF (keyword_set(rotate)) THEN FLAT = rotate(FLAT,1)
           dimFlat = size(FLAT)
           illum = dblarr(dimFlat(1))
           FOR kk = 0,dimFlat(1)-1 DO BEGIN
               illum(kk) = total(FLAT(kk,*))/float(dimFlat(2))
               FLAT(kk,*)= FLAT(kk,*)/illum(kk)             ;;just to make sure ...
           ENDFOR
;;;;;           FLAT     = FLAT/median(FLAT)
           PRINT,'% SPEX_REDUCE: Using '+log.flat+' for '+log.object
       ENDIF ELSE PRINT,'% SPEX_REDUCE: Using '+log.flat+' for '+log.object
       testbias = log.bias
       testflat = log.flat
       
;; ---------------------------------------------------------
;; --- READ IN OBJECT AND WAVE (EVERY TIME)
;;   
       IF i GT 0 THEN BEGIN
           OBJECT   = readfits(log.object,hobject)
           IF (keyword_set(rotate)) THEN OBJECT = rotate(OBJECT,1)
           IF ( n_elements(signed)  EQ 1) THEN SPEX_SIGNED,OBJECT
       ENDIF ELSE OBJECT = STANDARD  
       ;; behalte STANDARD aber behandle im folgenden STANDARD wie OBJECT
       WAVE     = readfits(log.wave  ,h)
       IF (keyword_set(rotate)) THEN WAVE = rotate(WAVE,1)
       IF ( n_elements(signed)  EQ 1) THEN SPEX_SIGNED,WAVE

;; ---------------------------------------------------------
;; --- BIAS AND FLATFIELD CORRECTIONS
;;   
    IF n_elements(overscan) EQ 0 THEN BEGIN
       OBJECT    = (OBJECT   - BIAS) / FLAT ; beinhaltet STANDARD !
       WAVE      = (WAVE     - BIAS) / FLAT
       PRINT,'% SPEX_REDUCE: Reduction done, starting extraction for '+STRING(i+1)
    ENDIF ELSE BEGIN
       OBJECT    = (OBJECT   - median([OBJECT(overscan(0):overscan(1),*), $
                                       OBJECT(overscan(2):overscan(3),*)])) / FLAT ; beinhaltet STANDARD !
       WAVE      = (WAVE     - median([  WAVE(overscan(0):overscan(1),*), $
                                         WAVE(overscan(2):overscan(3),*)])) / FLAT
       PRINT,'% SPEX_REDUCE: Reduction done, starting extraction for '+STRING(i+1)
    ENDELSE 
       
;; ---------------------------------------------------------
;; --- REDUCE OBJECT, STANDARD AND WAVE TO USABLE AREA
;;       
       OBJECT    =                OBJECT  (log.usable_begin:log.usable_end,*)
       WAVE      =                WAVE    (log.usable_begin:log.usable_end,*)
       IF i EQ 0 THEN STANDARD  = STANDARD(log.usable_begin:log.usable_end,*)
       IF i EQ 0 THEN illum     = illum   (log.usable_begin:log.usable_end)
       
;; ---------------------------------------------------------
;; --- LAGE DES SPEKTRUMS AUF DEM CHIP BESTIMMEN 
;;   
       PRINT,' '
       PRINT,' '
       PRINT,'% SPEX_REDUCE: Start reduction for: ',fxpar(hobject,'OBJECT')
       PRINT,' '
       PRINT,'% SPEX_REDUCE: Left   mouse click: spectrum position'
       PRINT,'% SPEX_REDUCE: Middle mouse click: skip this object'
       PRINT,'% SPEX_REDUCE: Right  mouse click: exit program now'
       PRINT,' '
       PRINT,' '
       
       siz   = size(OBJECT)
       xspec = fltarr(2*log.cuts_through_spec + 1)
       pixel = fltarr(2*log.cuts_through_spec + 1)
       IF i EQ 0 THEN howoften = 2*log.cuts_through_spec
       FOR m = 0, howoften  DO BEGIN
           IF m GT 0 THEN PRINT,'% SPEX_REDUCE: Left   mouse click: spectrum position'
           IF n_elements(center) EQ 0 THEN BEGIN 
               if(n_elements(xr) eq 0) then begin
                   y_min = min(OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),*)) * 0.8
                   IF (y_min LT 0.) THEN y_min = 0.
                   y_max = max(OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),*)) * 1.2
                   plot,OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),*),yrange=[y_min,y_max]
               endif
               if(n_elements(xr) ne 0) then begin
                  y_min = min(OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),xr(0):xr(1))) * 0.8
                  IF (y_min LT 0.) THEN y_min = 0.
                  y_max = max(OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),xr(0):xr(1))) * 1.2
                  plot,OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),*),yrange=[y_min,y_max],xrange=xr,xstyle=1
              endif
;               y_min = min(OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),*)) * 0.8
;               IF (y_min LT 0.) THEN y_min = 0.
;               y_max = max(OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),*)) * 1.2
;               plot,OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),*),yrange=[y_min,y_max]
           ENDIF ELSE BEGIN 
               y_min = min(OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),siz(2)/2-siz(2)*.3:siz(2)/2+siz(2)*.3)) * 0.8
               IF (y_min LT 0.) THEN y_min = 0.
               y_max = max(OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),siz(2)/2-siz(2)*.3:siz(2)/2+siz(2)*.3)) * 1.2
               if(n_elements(xr) eq 0) then plot,OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),siz(2)/2-siz(2)*.3:siz(2)/2+siz(2)*.3) $
                   ,yrange=[y_min,y_max]
               if(n_elements(xr) ne 0) then plot,OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),siz(2)/2-siz(2)*.3:siz(2)/2+siz(2)*.3) $
                   ,yrange=[y_min,y_max],xrange=xr,xstyle=1
;               plot,OBJECT(m*siz(1)/(2*log.cuts_through_spec+1),siz(2)/2-siz(2)*.3:siz(2)/2+siz(2)*.3) $
;                   ,yrange=[y_min,y_max]
           ENDELSE 
           pixel(m) = m*siz(1)/(2*log.cuts_through_spec+1)
           cursor,xcur,ycur     ; Klick verbessern durch Gaussfit           

           plots,xcur-log.specwidth/2.,(y_max+y_min)*0.3,color=20            ; extraction range
           plots,xcur+log.specwidth/2.,(y_max+y_min)*0.3,color=20,/continue
           plots,xcur-log.skyoff             ,(y_max+y_min)*0.2,color=80            ; sky range
           plots,xcur-log.skyoff-log.skywidth,(y_max+y_min)*0.2,color=80,/continue
           plots,xcur+log.skyoff             ,(y_max+y_min)*0.2,color=80            ; sky range
           plots,xcur+log.skyoff+log.skywidth,(y_max+y_min)*0.2,color=80,/continue
           wait,2

           IF n_elements(center) NE 0 THEN xcur = xcur+siz(2)/2-siz(2)*.3
           mouse=!mouse.button
           CASE mouse OF
               1: BEGIN
                   wait,0.3
                   xspec(m)=xcur
                   xg = findgen(2*fix(log.specwidth)+1)
                   yg = OBJECT(m*siz(1)/(2*log.cuts_through_spec+1), $
                               xspec(m)-log.specwidth:xspec(m)+log.specwidth)
                   gfit = gaussfit(xg,yg,a,nterms=4)
                   xspec(m) = a(1)+xspec(m)-log.specwidth-1
               END
               2: BEGIN
                   wait,0.3
                   print, ' '
                   print, '... skipping '+fxpar(hobject,'OBJECT')
                   print, ' '
                   IF i EQ 0 THEN BEGIN
                       IF exist(stdfluxtable) THEN useoldstandard = 1
                       skipstandard = 1
                       PRINT,'% SPEX_REDUCE: Sorry, I will need some clicks '+ $
                         'from you for the standard (determination of the trace of the spectrum)!'
                   ENDIF ELSE GOTO, endofloop 
               END
               4: BEGIN
                   wait,0.3
                   print, ' '
                   print, '... exiting !'
                   print, ' '
                   GOTO, ende
               END
           ENDCASE
       ENDFOR
       IF i EQ 0 AND log.cuts_through_spec GT 1 THEN $
         fitxspec = poly_fit (pixel, xspec, howoften/2 - 1,/double)
       xstar    = xspec(0)  
       howoften = 0       ; update only xspec(0) after first run
       
       ;; --- don't do things that already have been done 
       IF KEYWORD_SET(useoldstandard) AND i EQ 0 THEN GOTO, afterstandard
       IF skipstandard EQ 1 AND i EQ 0 THEN GOTO, afterstandard
       
;; ---------------------------------------------------------
;; --- EXTRACT OBJECT'S ( STANDARD'S if i = 0) SPECTRUM
;;   
; extrahieren , stern auf zeile xstar, spektrum 12 pixel breit, abstand
; spektrum-hintergrund 15, sky 50 pixel auf jeder seite
; defaults: laeuft dem spektrum auf dem chip nach (fswitch=1), fittet
; polynom (degree=3) plottet einiges, ; aber nicht alles (pswitch=2 ist ausfuehrlicher)
       ;; --- set color table for extract only once !
       obj_ex = SPEX_EXTRACT(OBJECT,   $
                        xstar,log.specwidth,log.skyoff,log.skywidth, $
                             specfit=1,filter=filter,pswitch=pswitch,step=5, $ 
                             level=level,lev_halbwertsbreite=lev_halbwertsbreite, $
                             fitcoeff=fitxspec)
       obj_ex = obj_ex/illum
       plot,obj_ex,xtitle='pixel',ytitle='counts',title='extracted & flatfielded spectrum of '$
         +fxpar(h,'OBJECT')     ; Rohspektrum
       PRINT,'% SPEX_REDUCE: Extracted spectrum of '+fxpar(h,'OBJECT')
       PRINT,'% SPEX_REDUCE: ready to continue? press return'
       read,cdummy
       
;; ---------------------------------------------------------
;; --- WELLENLAENGENKALIBRATION MACHEN 
;;
;;   
; wellenlaengenkalibration fuer die on-line reduktion nur einmal
; machen, danach aus dem file wave_fit.dat einlesen
       
       IF NOT KEYWORD_SET(useoldwave) THEN BEGIN        
           wave_ex = SPEX_EXTRACT(WAVE,   $
                             xstar,log.specwidth,0         ,0          , $
                                  fitcoeff=fitxspec,specfit=0,filter=0,pswitch=0,/nosky,$
                                 level=level,lev_halbwertsbreite=lev_halbwertsbreite) 
                                ; xstar ungefaehre Position des Sterns, oder 
                                ; einfach die Mitte des Chips quer zur
                                ; Dispersion , 12, 15, unwichtig hier,
                                ; /nosky unterdrueckt die
                                ; Himmelssubtraktion, die hier
                                ; unsinnig ist
           plot,wave_ex         ; extrahiertes Kalibrationsspektrum
           ;; --- resetting color table: spex_extract does that
           start_loc = start
           IF i GT 0 AND start NE 0 THEN start_loc = 0
           wcoeff =  SPEX_FIT_WAVE(wave_ex,log.wavedegree,start=start_loc,spectrum=spectrum,width=width,file=wave_fit,writeFlag=i) 
                                ; Kalibration, Anleitung on-line, 
                                ; Polynom vom Grad log.wavedegree,
                                ; wcoeff enthaelt Fit-Koeffizienten
           close,1
           wave_file = str_sep(log.wave,'/')
           openw,1,wave_file(n_elements(wave_file)-1)+'_wave' ; File fuer die Wellenlaengenkalibration
           printf,1,log.wavedegree
           printf,1,wcoeff
           close,1
           PRINT,'% SPEX_REDUCE: Startwellenlaenge: ',wcoeff(0)
       ENDIF ELSE BEGIN
           close,1
           wcoeff = dblarr(log.wavedegree+1) ; Grad des Wellenlaengen-Fit-Polynoms + 1
           wave_file = str_sep(log.wave,'/')
           openr,1,wave_file(n_elements(wave_file)-1)+'_wave' ; File fuer die Wellenlaengenkalibration
           readf,1,gradzurkontrolle
           readf,1,wcoeff
           close,1
           print,'% SPEX_REDUCE: Startwellenlaenge: ',wcoeff(0)
       ENDELSE
       
;; ---------------------------------------------------------
;; --- WAVE CALIBRATE OBJECT'S (STANDARD'S if i = 0) SPECTRUM
;;   
       ;; --- Stern wellenlaengenkalibrieren
       ss = size(obj_ex)
       xpixel = findgen(ss(1))
       xwave  = wcoeff(0)
       FOR k = 1,log.wavedegree DO xwave = xwave + wcoeff(k)*xpixel^k
       ;; --- xwave has changed dimension due to dimension of xpixel!
       plot,xwave,obj_ex,xtitle='wavelength',ytitle='counts',title=fxpar(hobject,'OBJECT')
       PRINT,'% SPEX_REDUCE: Lambda calibrated spectrum of '+fxpar(hobject,'OBJECT')
       PRINT,'% SPEX_REDUCE: return to continue...'
       read,cdummy

;; ---------------------------------------------------------
;; --- WRITE OBJECT'S (STANDARD'S if i = 0) SPECTRUM TO FILE 
;;   
       specFile = str_sep(log.object,'/')
       specFile = specFile(n_elements(specFile)-1)+'_spec'
       PRINT,'% SPEX_REDUCE: WRITING non-flux calibrated spectrum file '+specFile
       GET_LUN,unit
       OPENW,unit,specFile
       PRINTF, unit, ss(1)
       FOR c = 0, ss(1)-1 DO PRINTF, unit, xwave(c), obj_ex(c)
       FREE_LUN,unit
       
;; ---------------------------------------------------------
;; --- READ OBJECT'S (STANDARD'S if i = 0) SPECTRUM FROM FILE
;;   
       ;; Sprungaddresse falls 1. Eintrag im Logfile schonmal
       ;; abgearbeitet wurde, d.h. Standard zur fertig Verfuegung steht
       afterstandard  : $
       ;; Sprungaddresse falls ausschliesslich Flusskalibration gewuenscht
       fluxcalibstart : $
         IF KEYWORD_SET(fluxcalibonly) OR KEYWORD_SET(useoldstandard) THEN BEGIN 
           PRINT,'% SPEX_REDUCE: Skipped reduction & extraction part, instead: '
           hobject = headfits(log.object)
           specFile = str_sep(log.object,'/')
           specFile = specFile(n_elements(specFile)-1)+'_spec'
           PRINT,'% SPEX_REDUCE: READING non-flux calibrated spectrum file '+specFile
           GET_LUN,unit
           OPENR,unit,specFile
           READF, unit, dim
           xwave = dblarr(dim)
           obj_ex= dblarr(dim)
           FOR c = 0, dim-1 DO BEGIN
               READF, unit, xin, yin
               xwave(c) = xin
               obj_ex(c)= yin
           ENDFOR
           FREE_LUN,unit
       ENDIF
       
;; ---------------------------------------------------------
;; --- FLUX CALIBRATE OBJECT'S SPECTRUM (not for Standard of course) 
;;   
       IF EXIST(stdfluxtable) OR KEYWORD_SET(useoldstandard) THEN BEGIN
           
           IF i EQ 0 THEN BEGIN 
               ;; --- find and write OR read in flux calibration factors
               IF NOT KEYWORD_SET(useoldstandard) THEN BEGIN
                   xwavef = xwave
                   obj_exf= obj_ex
                   fcal     = SPEX_FLUXCAL(xwavef,obj_exf,file=stdfluxtable)
                   xwavecal = xwavef
                   ;; --- wegschreiben fcal xwavecal
                   dimen    = n_elements(xwavecal)
                   standardFile = str_sep(log.object,'/')
                   standardFile = standardFile(n_elements(standardFile)-1)+'_standard'
                   PRINT,'% SPEX_REDUCE: WRITING flux standard factors file '+standardFile
                   GET_LUN, unit
                   OPENW,unit,standardFile
                   PRINTF, unit, dimen
                   FOR c = 0, dimen-1 DO BEGIN
                       xin = xwavecal(c)
                       yin = fcal    (c)
                       PRINTF, unit, xin, yin
                   ENDFOR
                   FREE_LUN,unit
               ENDIF ELSE BEGIN
                   ;; --- einlesen fcal xwavecal
                   standardFile = str_sep(log.object,'/')
                   standardFile = standardFile(n_elements(standardFile)-1)+'_standard'
                   PRINT,'% SPEX_REDUCE: READING flux standard factors file '+standardFile
                   GET_LUN,unit
                   OPENR,unit,standardFile
                   READF, unit, dimen
                   xwavecal  = dblarr(dimen)
                   fcal      = dblarr(dimen)
                   FOR c = 0, dimen-1 DO BEGIN
                       READF, unit, xin, yin
                       xwavecal(c) = xin
                       fcal    (c) = yin
                   ENDFOR
                   FREE_LUN,unit
               ENDELSE
           ENDIF
           ;; --- apply flux calibration to object 
           obj_f = obj_ex*interpol(fcal,xwavecal,xwave)
           plot,xwave,obj_f,xtitle='wavelength',ytitle='flux',  $
             title=fxpar(hobject,'OBJECT')
           PRINT,'% SPEX_REDUCE: Flux calibrated spectrum of '+fxpar(hobject,'OBJECT')
           PRINT,'% SPEX_REDUCE: return to continue...'

           read,cdummy
       
           
;; ---------------------------------------------------------
;; --- WRITE FLUX CALIBRATED OBJECT'S SPECTRUM TO FILE
;;   
           fcspecFile = str_sep(log.object,'/')
           fcspecFile = fcspecFile(n_elements(fcspecFile)-1)+'_fcspec'
           PRINT,'% SPEX_REDUCE: WRITING flux calibrated spectrum file '+fcspecFile
           dimf = n_elements(obj_f)
           GET_LUN,unit
           OPENW,unit,fcspecFile
           PRINTF, unit, dimf
           FOR c = 0, dimf-1 DO BEGIN
               xout = xwave(c)
               yout = obj_f(c)
               PRINTF, unit, xout, yout
           ENDFOR
           FREE_LUN,unit
       ENDIF ELSE PRINT, '% SPEX_REDUCE: Skipped flux calibration part.'
       endofloop:
;; ======================== BIG LOOP END
   ENDFOR
;; ---------------------------------------------------------
;; --- WAR'S DAS ?
;;   
   PRINT, '% SPEX_REDUCE: Done.'
;   PRINT, 'Denke ueber schnell-wieder-Einlese-und-Darstell-Routine '
;   PRINT, 'der reduzierten Spektren nach ... / Modellspektren ... ?'
;   PRINT, 'Addition einzelner Spektren des gleichen Objekts ? '
   
;; ---------------------------------------------------------
;; --- END 
;;   
ende:   
END










